import string
from Tkinter import *

## CLASS INTEGER.FIELD:
## ====================
# Displays a text zone containing an integer and its associated +/- buttons

class IntegerField:

  ## CLASS ATTRIBUTES:
  ## -----------------
  # Pixmaps for plus & minus buttons
  plusImg = """#define plus_width 15
#define plus_height 7
static unsigned char plus_bits[] = {
   0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0xe0, 0x03, 0x80, 0x00, 0x80, 0x00,
   0x00, 0x00};
"""
  minusImg = """#define minus_width 15
#define minus_height 7
static unsigned char minus_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00};
"""
  
  ## METHOD __INIT__:
  ## ----------------
  # Creates an instance
  # parent  = the graphic container for the field
  # options = display options for the field

  def __init__(self, parent, **options):
    ## Get increment/decrement actions
    self.incrAction = None
    if options.has_key('incraction'):
      self.incrAction = options['incraction']
      del options['incraction']
    self.decrAction = None
    if options.has_key('decraction'):
      self.decrAction = options['decraction']
      del options['decraction']
    ## Surrounding frame
    self.fldFrame = Frame(parent)
    ## Is there a variable for the field?
    self.var = None
    entry = None
    if options.has_key('variable'):
      ## Then get it
      self.var = options['variable']
      ## Remove variable from options
      del options['variable']
      ## Width option is only for the entry
      width = 10
      if options.has_key('width'):
        width = options['width']
        del options['width']
      ## Create the entry
      entry = Entry(self.fldFrame, options, textvariable=self.var, width=width)
    ## If entry not built, build it
    if entry == None: entry = Entry(fldFrame, options)
    ## Set frame's options
    self.fldFrame.configure(options)
    ## Build +/- buttons
    btnFrm = Frame(self.fldFrame, options)
    btnFrm.pack(side=RIGHT)
    self.images={'plus':BitmapImage(data=IntegerField.plusImg), \
                 'minus':BitmapImage(data=IntegerField.minusImg)}
    plusBtn = Button(btnFrm, options, image=self.images['plus'], command=self.incrValue)
    plusBtn.pack(side=TOP)
    minusBtn = Button(btnFrm, options, image=self.images['minus'], command=self.decrValue)
    minusBtn.pack(side=BOTTOM)
    ## Insert entry
    entry.pack(side=LEFT, fill=Y)
    

  ## METHOD INCR.VALUE:
  ## ------------------
  # Increment value for field
  
  def incrValue(self):
    if self.var == None: return
    newValue = self.var.get() + 1
    if self.incrAction <> None:
      if not self.incrAction(newValue): return
    self.var.set(newValue)
    

  ## METHOD DECR.VALUE:
  ## ------------------
  # Decrement value for field
  
  def decrValue(self):
    if self.var == None: return
    newValue = self.var.get() - 1
    if self.decrAction <> None:
      if not self.decrAction(newValue): return
    self.var.set(newValue)
  
  
  ## METHOD PACK:
  ## ------------
  # Packs the field into its parent
  
  def pack(self, cnf={}, **options):
    for k in options.keys(): cnf[k] = options[k]
    self.fldFrame.pack(cnf)
  
  
  ## METHOD GRID:
  ## ------------
  # Put the field into its parent's grid
  
  def grid(self, cnf={}, **options):
    for k in options.keys(): cnf[k] = options[k]
    self.fldFrame.grid(cnf)
